/*
 * TismTool © Tis Veugen, 2010-2012
 * Simulation, Verification
 *
 * Tsm.java
 * version 4
 */
package TsmRTE.TsmGen;

import java.io.*;

 public class Tsm
 {
   public static final String NewLine = System.getProperty( "line.separator" );

   private static boolean      mbLogConsole = false;
   private static boolean      mbLogFile    = true;
   private static PrintWriter  mLogWriter   = null;
   private static int          mLogLevel    = 1;
   private static boolean      mbSeqFile    = true;
   private static PrintWriter  mSeqWriter   = null;
   private static int          mSeqClassId  = 0;

   public static void SetLogConsole( boolean bLogConsole )
   {
     mbLogConsole = bLogConsole;
   }

   public static void SetLogFile( boolean bLogFile )
   {
     mbLogFile = bLogFile;
   }

   public static void SetLogLevel( int iLevel )
   {
     mLogLevel = iLevel;
   }

   public static void SetSeqFile( boolean bSeqFile )
   {
     mbSeqFile = bSeqFile;
   }

   public static void Assert( boolean bCondition )
   {
     if ( ! bCondition )
     {
       System.out.println( "ASSERT @ " + Thread.currentThread().getStackTrace() );
       Tsm.Log( "ASSERT @ " + Thread.currentThread().getStackTrace() );
     }
   } 

   public static void Open()
   {
     if ( mbLogFile )
     {
       File fs = FreeTismFile( "Log" );
       try
       {
         mLogWriter = new PrintWriter( fs );
       }
       catch ( Exception e )
       {
         e.printStackTrace();
       }
     }

     if ( mbSeqFile )
     {
       File fs = FreeTismFile( "Seq" );
       try
       {
         mSeqWriter = new PrintWriter( fs );
       }
       catch ( Exception e )
       {
         e.printStackTrace();
       }
       Seq( "TismTool, Sequence Diagrams" );
     }
   }

   public static synchronized void Log( String s )
   {
     if ( null != mLogWriter )
     {
       mLogWriter.println( s );
       mLogWriter.flush();
     }

     if ( mbLogConsole )
     {
       System.out.println( s );
     }
   }

   public static void Lgl( int iLevel, String s )
   {
     if ( iLevel <= mLogLevel )
     {
       Log( s );
     }
   }

   public static synchronized int SeqClass( String s )
   {
     int Id;

     Id = ++mSeqClassId;

     Seq( "Class," + s + "," + Id );
     return Id;
   }

   public static synchronized void Seq( String s )
   {
     if ( null != mSeqWriter )
     {
       mSeqWriter.println( s );
       mSeqWriter.flush();
     }
   }

   public static void ShowException( String s, Exception e)
   {
     System.out.println( e.getMessage() );
     Tsm.Lgl( 1, e.getMessage() );
     Tsm.Lgl( 1, s + " : " + Tsm.StackTraceStrings(e) );
   }

   public static void Close()
   {
     if ( null != mLogWriter )
     {
       mLogWriter.close();
       mLogWriter = null;
     }

     if ( null != mSeqWriter )
     {
       mSeqWriter.close();
       mSeqWriter = null;
     }
   }

   private static File FreeTismFile( String sName )
   {
     File f = null;
     String s;
     int ix = 0;

     do
     {
       ix++;
       s = String.format( "Tism%s_%04d.txt", sName, ix );
       f = new File( s );
     } while ( f.exists() );

     return f;
   }

   public static String StackTraceStrings( Exception e )
   {
     StringBuilder sb = new StringBuilder();
     for ( StackTraceElement ste : e.getStackTrace() )
     {
       sb.append( ste.toString() );
       sb.append( Tsm.NewLine );
     }
     return sb.toString();
   }

   public static void ReportBox( String s )
   {
     System.out.println( "Report: " + s );
   }
 }

