/*
 * TismTool © Tis Veugen, 2010-2012
 * Application, Simulation, Verification
 *
 * TsmCompound.java
 * version 5
 */
package TsmRTE.TsmState;

import java.util.Vector;
import TsmRTE.TsmGen.TsmException;

public class TsmCompound extends TsmState
{
  protected Vector<TsmRegion>   mRegions = new Vector<TsmRegion>();

  @Override
  public void Initial( TsmHandler oHandler )
  {
    ExecuteRegions( oHandler, false );
  }

  @Override
  public void ExecuteTrigger( TsmHandler oHandler )
  {
    int ProspectSelf = GetProspect( oHandler.Trigger.PortId, oHandler.Trigger.Command );
    oHandler.Prospect |= ProspectSelf;

    ExecuteTriggerRegions( oHandler );
    
    if ( ! oHandler.bTaken )
    {
      // Process the trigger itself, if transition available.
      ExecuteTriggerCompound( oHandler );
    }
  }

  private void ExecuteTriggerRegions( TsmHandler oHandler )
  {
    ExecuteRegions( oHandler, true );
  }

  private void ExecuteRegions( TsmHandler oHandler, boolean bTrigger )
  {
    TsmHandler hAll = new TsmHandler();
    for ( TsmRegion tRegion : mRegions )
    {
      TsmHandler h;
      
      if ( bTrigger )
      {
        h = new TsmHandler( oHandler.Owner, oHandler.Trigger );
        h.Prospect = oHandler.Prospect;
        tRegion.ExecuteTrigger( h );
      }
      else
      {
        h = new TsmHandler( oHandler.Owner );
        h.Action = TsmHandlerAction.ENTER;
        tRegion.ExecuteHandler( h );
      }

      switch ( h.Action )
      {
        case TsmHandlerAction.TRANSITION:
          if ( TsmHandlerAction.hasTransition( hAll.Action ) )
          {
            throw new TsmException(
              String.format( "9610: Multiple region transitions @ %s", this ) );
          }
          hAll.Index = h.Index;
          break;
        case TsmHandlerAction.TACTRANSITION:
          if ( TsmHandlerAction.hasTransition( hAll.Action ) )
          {
            throw new TsmException(
              String.format( "9611: Multiple region transitions @ %s", this ) );
          }
          hAll.Index = h.Index;
          hAll.TacSubFsm = h.TacSubFsm;
          break;
        default:
          break;
      }
      hAll.Action |= h.Action;
      hAll.bTaken |= h.bTaken;
    }

    HandlerTakeOver( oHandler, hAll );
    oHandler.bTaken |= hAll.bTaken;
  }

  private void HandlerTakeOver( TsmHandler oHandler, TsmHandler hAll )
  {
    if ( (hAll.Action & TsmHandlerAction.TACTRANSITION) != 0 )
    {
      oHandler.Action    = TsmHandlerAction.TACTRANSITION;
      oHandler.Index     = hAll.Index;
      oHandler.TacSubFsm = hAll.TacSubFsm;
    }
    else
    if ( (hAll.Action & TsmHandlerAction.TRANSITION) != 0 )
    {
      oHandler.Action = TsmHandlerAction.TRANSITION;
      oHandler.Index  = hAll.Index;
    }
    else
    if ( (hAll.Action & TsmHandlerAction.OK) != 0 )
    {
      oHandler.Action = TsmHandlerAction.OK;
    }
    else
    if ( (hAll.Action & TsmHandlerAction.GUARD) != 0 )
    {
      oHandler.Action = TsmHandlerAction.GUARD;
    }
    else
    if ( hAll.Action == TsmHandlerAction.FINAL )
    {
      oHandler.Action = TsmHandlerAction.FINAL;
    }
    else
    if ( hAll.Action == TsmHandlerAction.NOK )
    {
      oHandler.Action = TsmHandlerAction.NOK;
    }
    else
    if ( TsmHandlerAction.isFinalNok( hAll.Action ) )
    {
      oHandler.Action = TsmHandlerAction.NOK;
    }
    else
    {
      oHandler.Action = TsmHandlerAction.NOK;
      throw new TsmException(
        String.format( "TismBUG, HandlerTakeOver action @ %s", this ) );
    }
  }

  public void ExecuteTriggerCompound( TsmHandler oHandler )
  {
    // Override for all the compound triggers
    oHandler.Action = TsmHandlerAction.NOK;
  }

  public void AtMyExit( TsmHandler oHandler )
  {
  }

  public void ExitRegions( TsmHandler oHandler )
  {
    for ( TsmRegion tRegion : mRegions )
    {
      tRegion.AtExit( oHandler );
    }
  }

  @Override
  public void AtExit( TsmHandler oHandler )
  {
    ExitRegions( oHandler );
    AtMyExit( oHandler );
  }

  @Override
  public String TsmSnap()
  {
    StringBuilder sb = new StringBuilder();
    sb.append( "[" + this.TsmName + ":" );
    for ( TsmRegion tRegion : mRegions )
    {
      sb.append( "|" + tRegion.TsmSnap() );
    }
    return sb.toString() + "]";
  }

  @Override
  public void TsmCheckVisit()
  {
    for ( TsmRegion tRegion : mRegions )
    {
      tRegion.TsmCheckVisit();
    }
  }
}
