/*
 * TismTool © Tis Veugen, 2010-2012
 * Application, Simulation, Verification
 *
 * TsmRegion.java
 * version 3
 */
package TsmRTE.TsmState;

import TsmRTE.TsmGen.TsmException;

public class TsmRegion extends TsmState
{
  protected int         mNrStates = 0;
  protected TsmState [] mStates = null;
  protected TsmState    mCurrentState = null;
  protected boolean     mbFinal = false;

  /**
   * This function should be overridden, or never be called.
   */
  @Override
  public void Initial( TsmHandler oHandler )
  {
    throw new TsmException(
      String.format( "9600: Region without Initial state @ %s", this ) );
  }

  @Override
  public void ExecuteTrigger( TsmHandler oHandler )
  {
    if ( mbFinal )
    {
      oHandler.Action = TsmHandlerAction.FINAL;
      return;
    }

    int Prospect = GetProspect( oHandler.Trigger.PortId, oHandler.Trigger.Command );
    switch ( Prospect )
    {
    case TsmProspect.NOK :
      oHandler.Action = TsmHandlerAction.NOK;
      return;
    case TsmProspect.ANY :
      if ( (oHandler.Prospect & TsmProspect.OK) != 0 )
      { /*
         * Execute the trigger at the higher level.
         */
        oHandler.Action = TsmHandlerAction.NOK;
        return;
      }
      break;
    default :
      break;
    }

    mCurrentState.ExecuteTrigger( oHandler );
    mCurrentState.ExecuteHandler( oHandler );
    ExecuteHandler( oHandler );
    mbFinal = ( oHandler.Action == TsmHandlerAction.FINAL );
  }

  @Override
  public void ExecuteTriggerless( TsmHandler oHandler )
  {
    // Prevent redefining oHandler.Action
  }

  @Override
  public void AtExit( TsmHandler oHandler )
  {
    if ( null != mCurrentState )
    {
      mCurrentState.AtExit( oHandler );
      Reset();
    }
  }

  public void Reset()
  {
    mCurrentState = null;
    mbFinal = false;
  }

  @Override
  public String TsmSnap()
  {
    if ( null == mCurrentState )
    {
      return "";
    }
    return mCurrentState.TsmSnap();
  }

  @Override
  public void TsmCheckVisit()
  {
    for ( int i=0; i<mNrStates; i++ )
    {
      mStates[i].TsmCheckVisit();
    }
  }
}
