/*
 * TismTool © Tis Veugen, 2010-2012
 * Simulation, Verification
 *
 * TsmState.java
 * version 2
 */
package TsmRTE.TsmState;

import TsmRTE.TsmGen.*;
import TsmRTE.TsmSupervisor.TsmRecorder;

public class TsmState extends TsmBase
{
  public boolean          bVisit = false;
  public int              mNrTransitions = 0;
  public TsmTransition [] mTransitions = null;

  public void AtEntry( TsmHandler oHandler )
  {
  }

  public void AtExit( TsmHandler oHandler )
  {
  }

  public void Initial( TsmHandler oHandler )
  {
  }

  public void Entrypoint( TsmHandler oHandler )
  {
  }

  public void ExecuteTrigger( TsmHandler oHandler )
  {
    // N.B. Some states may have no triggers
    oHandler.Action = TsmHandlerAction.NOK;
  }

  public void ExecuteTriggerless( TsmHandler oHandler )
  {
    oHandler.Action = TsmHandlerAction.NOK;
  }

  public boolean ExecuteTransition( TsmHandler oHandler )
  {
    return true;
  }

  public void ExecuteHandler( TsmHandler oHandler )
  {
    boolean bReady = false;

    do
    {
      switch ( oHandler.Action )
      {
      case TsmHandlerAction.TRANSITION:
      case TsmHandlerAction.TACTRANSITION:
        bReady = ExecuteTransition( oHandler );
        break;
      case TsmHandlerAction.EXITPOINT:
        oHandler.Action = TsmHandlerAction.TRANSITION;
        bReady = true;
        break;
      case TsmHandlerAction.ENTER:
        AtEntry( oHandler );
        Initial( oHandler );
        break;
      case TsmHandlerAction.INITIAL:
        Initial( oHandler );
        break;
      case TsmHandlerAction.ENTRYPOINT:
        AtEntry( oHandler );
        Entrypoint( oHandler );
        break;
      case TsmHandlerAction.OK:
      case TsmHandlerAction.GUARD:
      case TsmHandlerAction.NOK:
        bReady = true;
        break;
      case TsmHandlerAction.FINAL:
      case TsmHandlerAction.QUIT:
        ExecuteTriggerless( oHandler );
        bReady = true;
        break;
      default :
        throw new TsmException(
          String.format( "TsmHandlerAction '%s' @ '%s", oHandler.Action, this ) );
        //bReady = true;
        //break;
      }
    } while ( ! bReady );
  }

  public int GetProspect( int PortId, int Command )
  {
    return TsmProspect.NOK;
  }

  public String TsmSnap()
  {
    return this.TsmName;
  }

  public void TsmCheckVisit()
  {
    if ( ! bVisit )
    {
      TsmRecorder.UnvisitedState( this.FullName() );
    }

    for ( int i=0; i<mNrTransitions; i++ )
    {
      if ( ! mTransitions[i].bVisit )
      {
        TsmRecorder.UnvisitedTransition( this.FullName(), mTransitions[i].FromTo );
      }
    }
  }
}
