/*
 * TismTool © Tis Veugen, 2010-2012
 * Simulation, Verification
 *
 * TsmRecorder.java
 * version 8
 */
package TsmRTE.TsmSupervisor;

import java.util.Vector;
import TsmRTE.TsmComponent.TsmClass;
import TsmRTE.TsmComponent.TsmPort;
import TsmRTE.TsmGen.Tsm;

public class TsmRecorder
{
  public static int           NumberOfUnhandledFunctions = 0;
  public static int           NumberOfUnvisitedStates = 0;
  public static int           NumberOfUnvisitedTransitions = 0;
  public static int           NumberOfSyncReturnErrors = 0;
  public static int           NumberOfQueueOverflows = 0;
  
  private static Vector<String> Records;
  private static int            NrCalls;
  private static boolean        bEnable;
  private static String         Separator = "============================================================";

  static
  {
    Reset();
  }

  public static int getNumberOfRecords()
  {
    return Records.size();
  }

  public static int getNumberOfExtCalls()
  {
    return NrCalls;
  }

  public static void Reset()
  {
    Records = new Vector<String>();
    NrCalls = 0;
    bEnable = false;
  }

  public static void Enable()
  {
    bEnable = true;
  }

  public static void AddNative( String sNative )
  {
    String s = "NaTr: " + sNative;
    Records.add( s );
  }

  public static void AddCall( String sCall, TsmClass oSource, TsmPort oPort )
  {
    TsmClass  lTarget = oPort.Target;
    String    sSource = "'foreign'";
    boolean   bSourceIsMock = false;
    int       iSourceSeqId = -1;
    if ( ( oSource != null ) && ( oSource.SeqId > 0 ) )
    {
      sSource = oSource.FullName();
      bSourceIsMock = oSource.IsMock;
      iSourceSeqId = oSource.SeqId;
    }

    String s = "Call: " + sCall + " @ " + sSource + " -> " + lTarget.FullName();
    Records.add( s );
    if ( bSourceIsMock || lTarget.IsMock )
    {
      NrCalls++;
    }
    if ( iSourceSeqId >= 0 )
    {
      Tsm.Seq( "C," + sCall + "," + iSourceSeqId + "," + lTarget.SeqId + "," + oPort.ToSeq() );
    }
  }

  public static void AddStates( String sStates )
  {
    String s = "States: " + sStates;
    Records.add( s );
  }

  public static void UnhandledFunction( String sEvent )
  {
    NumberOfUnhandledFunctions++;
    Show( "**** UNHANDLED FUNCTION : " + sEvent );
  }

  public static void UnvisitedState( String sEvent )
  {
    StringBuilder sb = new StringBuilder();

    NumberOfUnvisitedStates++;
    sb.append( Separator );
    sb.append( Tsm.NewLine );
    sb.append( "Unvisited state: " + sEvent );
    sb.append( Tsm.NewLine );
    Tsm.Log( sb.toString() );
  }

  public static void UnvisitedTransition( String sState, String sEvent )
  {
    StringBuilder sb = new StringBuilder();

    NumberOfUnvisitedTransitions++;
    sb.append( Separator );
    sb.append( Tsm.NewLine );
    sb.append( "@ state: " + sState );
    sb.append( Tsm.NewLine );
    sb.append( "Unvisited transition: " + sEvent );
    sb.append( Tsm.NewLine );
    Tsm.Log( sb.toString() );
  }

  public static void SyncReturnError( String sEvent )
  {
    NumberOfSyncReturnErrors++;
    Show( sEvent );
  }

  public static void QueueOverflow()
  {
    NumberOfQueueOverflows++;
    Show( "**** QUEUE OVERFLOW " );
  }

  public static void Show( String sEvent )
  {
    if ( bEnable )
    {
      StringBuilder sb = new StringBuilder();
      sb.append( Separator );
      sb.append( Tsm.NewLine );
      sb.append( sEvent );
      sb.append( Tsm.NewLine );

      for ( String s : Records )
      {
        sb.append( s );
        sb.append( Tsm.NewLine );
      }

      sb.append( TsmSupervisor.SnappedClasses() );
      Tsm.Log( sb.toString() );
    }
    Tsm.Seq( "X," + sEvent );
  }

  public static void Show( String sEvent, int iNrRecords )
  {
    if ( bEnable )
    {
      StringBuilder sb = new StringBuilder();
      sb.append( Separator );
      sb.append( Tsm.NewLine );
      sb.append( sEvent );
      sb.append( Tsm.NewLine );

      for ( int i=0; i<iNrRecords; i++ )
      {
        sb.append( Records.get(i) );
        sb.append( Tsm.NewLine );
      }
      sb.append( "=== start of cycle ===" );
      sb.append( Tsm.NewLine );
      for ( int i=iNrRecords; i<Records.size(); i++ )
      {
        sb.append( Records.get(i) );
        sb.append( Tsm.NewLine );
      }

      sb.append( "Classes:" );
      sb.append( Tsm.NewLine );
      sb.append( TsmSupervisor.GetCurrentClasses() );

      sb.append( "Scenario:" );
      sb.append( Tsm.NewLine );
      sb.append( TsmSupervisor.GetCurrentScenario() );

      Tsm.Log( sb.toString() );
    }
    Tsm.Seq( "X," + sEvent );
  }
}
